//
//  GameScene.swift
//  RestoringLastPosition
//
//  Created by Lunapip on 31/08/2017.
//  Copyright © 2017 Lunapip. All rights reserved.
//

import SpriteKit
import GameplayKit

// These are the physics categories so we can check which
// objects have made contact.
struct PhysicsCategory {
    static let None                 : UInt32 = 0
    static let All                  : UInt32 = UInt32.max
    static let Player               : UInt32 = 0x1 << 1
    static let Bomb                 : UInt32 = 0x1 << 2
    static let RestorePoint         : UInt32 = 0x1 << 3
}

// This struct is available globablly. Here we are just using
// it to store the lastRestorePoint position.
struct Game {
    static var lastRestorePoint: CGPoint?
}

// The RestorePoint class sets up its physics information
// and hides itelf when the game is playing.
class RestorePoint: SKSpriteNode {
    
    required init?(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)

        isHidden = false
        
        physicsBody?.categoryBitMask = PhysicsCategory.RestorePoint
        physicsBody?.contactTestBitMask = PhysicsCategory.Player
        physicsBody?.collisionBitMask = PhysicsCategory.None
        physicsBody?.isDynamic = false
        physicsBody?.affectedByGravity = false
    }
    
}

// The Bomb class sets up its physics information.
class Bomb: SKSpriteNode {
    
    required init?(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)
        
        physicsBody?.categoryBitMask = PhysicsCategory.Bomb
        physicsBody?.contactTestBitMask = PhysicsCategory.Player
        physicsBody?.collisionBitMask = PhysicsCategory.None
        physicsBody?.isDynamic = false
        physicsBody?.affectedByGravity = false
    }
    
}

class GameScene: SKScene, SKPhysicsContactDelegate {
    
    var player: SKSpriteNode!
    
    override func didMove(to view: SKView) {
        super.didMove(to: view)
        
        // We set the contactDelegate of the physics world to the game scene
        // so that we can check when objects make contact with each other.
        physicsWorld.contactDelegate = self
        
        // Get the Player sprite from the scene and store in the player
        // variable.
        player = childNode(withName: "Player") as! SKSpriteNode
        
        // Define the physics properties of the player sprite.
        player.physicsBody?.categoryBitMask = PhysicsCategory.Player
        player.physicsBody?.contactTestBitMask = PhysicsCategory.RestorePoint
        player.physicsBody?.collisionBitMask = PhysicsCategory.None
        
        // Check the lastRestorePoint
        if let lastRestorePoint = Game.lastRestorePoint {
            player.position = lastRestorePoint
        } else {
            // The lastRestorePoint variable is not set. Either
            // this is the first time playing or the player has
            // died before reaching a restore point.
        }
    }
    
    func didBegin(_ contact: SKPhysicsContact) {
        var firstBody: SKPhysicsBody
        var secondBody: SKPhysicsBody
        
        // Ensure lowest category is always stored in the first body.
        if contact.bodyA.categoryBitMask < contact.bodyB.categoryBitMask {
            firstBody = contact.bodyA
            secondBody = contact.bodyB
        } else {
            firstBody = contact.bodyB
            secondBody = contact.bodyA
        }
        
        if firstBody.categoryBitMask == PhysicsCategory.Player && secondBody.categoryBitMask == PhysicsCategory.RestorePoint {
            
            Game.lastRestorePoint = player.position
        }
        
        if firstBody.categoryBitMask == PhysicsCategory.Player && secondBody.categoryBitMask == PhysicsCategory.Bomb {
            
            let transition = SKTransition.doorway(withDuration: 0.5)
            if let gameScene = SKScene(fileNamed: "GameScene") {
                self.view?.presentScene(gameScene, transition: transition)
            }
        }
    }
    
    deinit {
        print("The previous game scene has been deinited👍")
    }
    
    override func update(_ currentTime: TimeInterval) {
        // Called before each frame is rendered
        
        if let camera = self.camera {
            camera.position.x = player.position.x
        }
        
        player.physicsBody?.applyForce(CGVector(dx: 20, dy: 0))
    }
}
